@@ -1,3 +1,6 @@
+#' @importFrom nlme lme lmeControl pdIdent varIdent ranef pdBlocked pdDiag pdSymm
+#' @importFrom utils tail
+#' @importFrom stats residuals
 VC.est.random <-
 function( data,
            IxR = has.repl(data), linked = IxR,
@@ -19,7 +22,7 @@ if( missing(MxI) ) MxI <- matrix
 if( missing(IxR) ) IxR <- linked
 
 # Package needed for the fitting of the models
-require( nlme )
+# require( nlme )
 
 # Make all variables local to the function environment
 meth <- factor(data$meth)
@@ -173,7 +176,7 @@ names( Bias ) <- levels( meth )
 Mu <- summ[grep("item",rownames(summ)),1]
 
 # The two-way random interactions
-vc <- nlme:::VarCorr( m1 )
+vc <- nlme::VarCorr( m1 )
 xi <- as.numeric( vc[grep("^meth(.+)",rownames(vc)),2][1] )
 
 if ( varMxI & MxI ) tau <- as.numeric( tail(vc[grep("^meth(.+)",rownames(vc)),2], nlevels(meth)) )
