@@ -1,16 +1,31 @@
+#' Subset an MCmcmc object
+#'
+#' Subset an MCmcmc object
+#'
+#' @param x object to be subsetted.
+#' @param subset Numerical, character or list giving the variables to keep.
+#'                If numerical, the variables in the \code{MCmcmc} object with
+#'                these numbers are selected. If character, each element of the
+#'                character vector is "grep"ed against the variable names, and
+#'                the matches are selected to the subset. If a list each element
+#'                is used in turn, numerical and character elements can be mixed.
+#' @param allow.repl Logical. Should duplicate columns be allowed in the result?
+#' @param chains Numerical vector giving the number of the chains to keep.
+#' @param ... further arguments to be passed to or from other methods.
+#'
+#' @rdname subset
+#' @export
 subset.MCmcmc <-
-function( x, subset=NULL, allow.repl=FALSE, chains=NULL, ... )
-{
+function( x, subset=NULL, allow.repl=FALSE, chains=NULL, ... ) {
 # A small utility function
-char.2.num <-
-function( x, sbs )
-{
-sb <- numeric(0)
-for( i in 1:length(sbs))
-    sb <- c(sb, grep( sbs[i], varnames(x) ) )
-sb
-}
-
+  char.2.num <-
+  function( x, sbs ) {
+  sb <- numeric(0)
+  for( i in 1:length(sbs))
+      sb <- c(sb, grep( sbs[i], varnames(x) ) )
+  sb
+  }
+ 
 if( is.numeric(chains) )
   {
   chains <- (1:attributes(x)$mcmc.par$n.chains)[chains]
